@@ -15,14 +15,16 @@
 
 import anyjson
 import eventlet
+import types
 
 from muranoconductor.openstack.common import log as logging
 import muranoconductor.helpers
 from command import CommandBase
 import muranoconductor.config
-from heatclient.client import Client
 import heatclient.exc
 from keystoneclient.v2_0 import client as ksclient
+from heatclient import client as heat_client
+from quantumclient.v2_0 import client as quantum_client
 
 log = logging.getLogger(__name__)
 
@@ -36,6 +38,7 @@ class HeatExecutor(CommandBase):
 
         keystone_settings = muranoconductor.config.CONF.keystone
         heat_settings = muranoconductor.config.CONF.heat
+        quantum_settings = muranoconductor.config.CONF.quantum
 
         client = ksclient.Client(
             endpoint=keystone_settings.auth_url,
@@ -54,9 +57,20 @@ class HeatExecutor(CommandBase):
             service_type='orchestration',
             endpoint_type=heat_settings.endpoint_type)
 
-        self._heat_client = Client(
+        quantum_url = client.service_catalog.url_for(
+            service_type='network',
+            endpoint_type=quantum_settings.endpoint_type)
+
+        self._quantum_client = quantum_client.Client(
+            endpoint_url=quantum_url,
+            token=token,
+            ca_cert=quantum_settings.ca_cert or None,
+            insecure=quantum_settings.insecure)
+
+        self._heat_client = heat_client.Client(
             '1',
             heat_url,
+            token_only=True,
             token=client.auth_token,
             ca_file=heat_settings.ca_file or None,
             cert_file=heat_settings.cert_file or None,
@@ -80,7 +94,8 @@ class HeatExecutor(CommandBase):
     def _execute_create_update(self, template, mappings, arguments, callback):
         with open('data/templates/cf/%s.template' % template) as template_file:
             template_data = template_file.read()
-
+        if not 'externalNetworkId' in mappings:
+            mappings['externalNetworkId'] = self._get_external_network_id()
         template_data = muranoconductor.helpers.transform_json(
             anyjson.loads(template_data), mappings)
 
@@ -95,11 +110,28 @@ class HeatExecutor(CommandBase):
             'callback': callback
         })
 
+    def _get_external_network_id(self):
+        log.info('Fetching the list of external networks...')
+        ext_nets = self._quantum_client.list_networks(
+            **{'router:external': True}).get('networks')
+        log.debug(ext_nets)
+        if ext_nets and len(ext_nets) > 0:
+            if len(ext_nets) > 1:
+                log.warning(
+                    'Multiple external networks found, will use the first one')
+            net = ext_nets[0]
+            return net.get('id')
+        else:
+            log.error('No external networks found!')
+            return None
+
     def has_pending_commands(self):
         return len(self._update_pending_list) + len(
             self._delete_pending_list) > 0
 
     def execute_pending(self):
+        # wait for the stack not to be IN_PROGRESS
+        self._wait_state(lambda status: True)
         r1 = self._execute_pending_updates()
         r2 = self._execute_pending_deletes()
         return r1 or r2
@@ -109,9 +141,6 @@ class HeatExecutor(CommandBase):
             return False
 
         try:
-            # wait for the stack not to be IN_PROGRESS
-            self._wait_state(lambda status: True)
-
             template, arguments = self._get_current_template()
             stack_exists = (template != {})
             # do not need to merge with current stack cause we rebuilding it
@@ -133,14 +162,11 @@ class HeatExecutor(CommandBase):
                     stack_id=self._stack,
                     parameters=arguments,
                     template=template)
-
                 log.debug(
                     'Waiting for the stack {0} to be update'.format(
                         self._stack))
-
                 outs = self._wait_state(
                     lambda status: status == 'UPDATE_COMPLETE')
-
                 log.info('Stack {0} updated'.format(self._stack))
             else:
                 self._heat_client.stacks.create(
@@ -151,10 +177,8 @@ class HeatExecutor(CommandBase):
 
                 log.debug('Waiting for the stack {0} to be create'.format(
                     self._stack))
-
                 outs = self._wait_state(
                     lambda status: status == 'CREATE_COMPLETE')
-
                 log.info('Stack {0} created'.format(self._stack))
 
             pending_list = self._update_pending_list
@@ -176,9 +200,6 @@ class HeatExecutor(CommandBase):
 
         log.debug('Deleting stack {0}'.format(self._stack))
         try:
-            # wait for the stack not to be IN_PROGRESS
-            self._wait_state(lambda status: True)
-
             self._heat_client.stacks.delete(
                 stack_id=self._stack)
             log.debug(
@@ -186,8 +207,6 @@ class HeatExecutor(CommandBase):
             self._wait_state(
                 lambda status: status in ('DELETE_COMPLETE', 'NOT_FOUND'))
             log.info('Stack {0} deleted'.format(self._stack))
-        except heatclient.exc.HTTPNotFound:
-            log.warn('Deleting non-existing stack {0}'.format(self._stack))
         except Exception as ex:
             log.exception(ex)
 
